import json
import os
import magic
from io import BytesIO

from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.files.base import ContentFile
from django.core.files.storage import default_storage as storage
from django.db import models
from django.db.models.fields.related import ForeignObjectRel
from ..fields import JSONField
from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
from hvad.models import TranslatableModel, TranslatedFields
from PIL import Image


class BaseMediaFolder(TranslatableModel):
    translations = TranslatedFields(
        description=models.CharField(max_length=200, blank=True, null=True),
        title=models.CharField(max_length=200, blank=True, null=True),
    )
    slug = models.SlugField()
    creation_date = models.DateTimeField(auto_now_add=True)
    last_modified = models.DateTimeField(auto_now=True)
    icon = models.ForeignKey(
        "camomilla.Media",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        verbose_name=_("Image cover"),
    )
    path = models.TextField(blank=True, null=True)
    updir = models.ForeignKey(
        "self",
        on_delete=models.CASCADE,
        related_name="child_folders",
        null=True,
        blank=True,
    )

    class Meta:
        abstract = True

    def update_childs(self):
        for folder in self.child_folders.all():
            folder.save()

    def save(self, *args, **kwargs):
        if self.updir:
            if self.updir.id == self.id:
                raise ValidationError({"updir": "Unvalid parent"})
            self.path = "{0}/{1}".format(self.updir.path, self.slug)

        else:
            self.path = "/{0}".format(self.slug)

        super(BaseMediaFolder, self).save(*args, **kwargs)
        self.update_childs()

    def __str__(self):
        to_string = self.slug
        if self.title:
            to_string += " - " + self.title
        return to_string


class MediaFolder(BaseMediaFolder):
    translations = TranslatedFields()


class Media(TranslatableModel):
    translations = TranslatedFields(
        alt_text=models.CharField(max_length=200, blank=True, null=True),
        title=models.CharField(max_length=200, blank=True, null=True),
        description=models.TextField(blank=True, null=True),
    )
    file = models.FileField()
    thumbnail = models.ImageField(
        upload_to=getattr(settings, "THUMB_FOLDER", "thumbnails"),
        max_length=500,
        null=True,
        blank=True,
    )
    created = models.DateTimeField(auto_now=True)
    name = models.CharField(max_length=200, blank=True, null=True)
    size = models.IntegerField(default=0, blank=True, null=True)
    mime_type = models.CharField(max_length=128, blank=True, null=True)
    image_props = JSONField(default=dict, blank=True)
    folder = models.ForeignKey(
        MediaFolder,
        null=True,
        blank=True,
        related_name="media_folder",
        on_delete=models.CASCADE,
    )

    @property
    def path(self):
        return "%s/%s" % (self.folder.path, self.name)

    @property
    def is_image(self):
        return self.mime_type and self.mime_type.startswith("image")

    def image_preview(self):
        if self.file:
            return mark_safe('<img src="{0}" />'.format(self.file.url))

    def image_thumb_preview(self):
        if self.thumbnail:
            return mark_safe('<img src="{0}" />'.format(self.thumbnail.url))

    image_preview.short_description = _("Preview")
    image_thumb_preview.short_description = _("Thumbnail")

    class Meta:
        ordering = ["-pk"]

    def regenerate_thumbnail(self):
        if self.file:
            self._make_thumbnail()

    def get_foreign_fields(self):
        return [
            field.get_accessor_name()
            for field in self._meta.get_fields()
            if issubclass(type(field), ForeignObjectRel)
        ]

    @property
    def json_repr(self):
        json_r = {
            "id": self.pk,
            "thumbnail": "" if not self.is_image else self.thumbnail.url,
            "label": self.__str__(),
        }
        return json.dumps(json_r)

    def _make_thumbnail(self):
        try:
            fh = storage.open(self.file.name, "rb")
            self.mime_type = magic.from_buffer(fh.read(2048), mime=True)
        except FileNotFoundError as ex:
            print(ex)
            self.image_props = {}
            self.mime_type = ""
            return False
        try:
            orig_image = Image.open(fh)
            image = orig_image.copy()
            self.image_props = {
                "width": orig_image.width,
                "height": orig_image.height,
                "format": orig_image.format,
                "mode": orig_image.mode,
            }
        except Exception as ex:
            print(ex)
            return False

        try:
            image.thumbnail(
                (
                    getattr(settings, "CAMOMILLA_THUMBNAIL_WIDTH", 50),
                    getattr(settings, "CAMOMILLA_THUMBNAIL_HEIGHT", 50),
                ),
                Image.ANTIALIAS,
            )
            fh.close()

            # Path to save to, name, and extension
            thumb_name, thumb_extension = os.path.splitext(self.file.name)
            thumb_extension = thumb_extension.lower()

            thumb_filename = thumb_name + "_thumb" + thumb_extension

            temp_thumb = BytesIO()
            image.save(temp_thumb, "PNG", optimize=True)
            temp_thumb.seek(0)

            # Load a ContentFile into the thumbnail field so it gets saved
            self.thumbnail.save(
                thumb_filename, ContentFile(temp_thumb.read()), save=False
            )
            temp_thumb.close()
        except Exception:
            return False

        return True

    def _remove_file(self):
        if self.file:
            file_to_remove = os.path.join(settings.MEDIA_ROOT, self.file.name)
            if os.path.isfile(file_to_remove):
                os.remove(file_to_remove)

    def _remove_thumbnail(self):
        if self.thumbnail:
            file_to_remove = os.path.join(settings.MEDIA_ROOT, self.thumbnail.name)
            if os.path.isfile(file_to_remove):
                os.remove(file_to_remove)

    def _get_file_size(self):
        if self.file:
            file_to_calc = os.path.join(settings.MEDIA_ROOT, self.file.name)
            if os.path.isfile(file_to_calc):
                return self.file.size
            else:
                return 0

    def __str__(self):
        if self.name:
            return self.name
        return self.file.name


@receiver(post_save, sender=Media, dispatch_uid="make thumbnails")
def update_media(sender, instance, **kwargs):
    instance._remove_thumbnail()
    instance._make_thumbnail()
    Media.objects.filter(pk=instance.pk).update(
        size=instance._get_file_size(),
        thumbnail=instance.thumbnail,
        mime_type=instance.mime_type,
        image_props=instance.image_props,
    )


@receiver(pre_delete, sender=Media, dispatch_uid="make thumbnails")
def delete_media_files(sender, instance, **kwargs):
    instance._remove_thumbnail()
    instance._remove_file()



# Generated by Django 2.2.25 on 2022-02-11 16:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("camomilla", "0006_auto_20220103_1845"),
    ]

    operations = [
        migrations.AlterField(
            model_name="articletranslation",
            name="permalink",
            field=models.SlugField(max_length=200),
        ),
    ]



# Generated by Django 2.2.17 on 2021-01-30 16:10

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("camomilla", "0002_auto_20200214_1127"),
    ]

    operations = [
        migrations.AddField(
            model_name="mediafolder",
            name="path",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="mediafolder",
            name="updir",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="child_folders",
                to="camomilla.MediaFolder",
            ),
        ),
    ]



from .views import (
    ArticleViewSet,
    CamomillaObtainAuthToken,
    CategoryViewSet,
    MediaFolderViewSet,
)
from .views import TagViewSet, ContentViewSet, MediaViewSet, PermissionViewSet
from .views import PageViewSet, LanguageViewSet, UserViewSet

from django.urls import include, path
from django.shortcuts import redirect

from rest_framework import routers

router = routers.DefaultRouter()

router.register(r"tags", TagViewSet, "camomilla-tags")
router.register(r"categories", CategoryViewSet, "camomilla-categories")
router.register(r"articles", ArticleViewSet, "camomilla-articles")
router.register(r"contents", ContentViewSet, "camomilla-content")
router.register(r"media", MediaViewSet, "camomilla-media")
router.register(r"media-folders", MediaFolderViewSet, "camomilla-media_folders")
router.register(r"pages", PageViewSet, "camomilla-pages")
router.register(r"sitemap", PageViewSet, "camomilla-sitemap")
router.register(r"users", UserViewSet, "camomilla-users")
router.register(r"permissions", PermissionViewSet, "camomilla-permissions")


urlpatterns = [
    path("", include(router.urls)),
    path(
        "profiles/me/", lambda _: redirect("../../users/current/"), name="profiles-me"
    ),
    path("token-auth/", CamomillaObtainAuthToken.as_view(), name="api_token"),
    path("languages/", LanguageViewSet.as_view(), name="get_languages"),
]



from django.db import models
from hvad.models import TranslatedFields
from .mixins import SeoMixin
from ..utils import get_page


class BasePage(SeoMixin):
    identifier = models.CharField(max_length=200, unique=True)
    translations = TranslatedFields()

    class Meta:
        abstract = True
        verbose_name = "Page"
        verbose_name_plural = "Pages"

    @classmethod
    def get(model, request, **kwargs):
        return get_page(request, **kwargs)

    def alternate_urls(self, request):
        from djlotrek.context_processors import alternate_seo_url

        return alternate_seo_url(request)

    def __str__(self):
        return self.identifier


class Page(BasePage):
    translations = TranslatedFields()



from collections import OrderedDict
from rest_framework import serializers


class RelatedField(serializers.PrimaryKeyRelatedField):
    def __init__(self, **kwargs):
        self.serializer = kwargs.pop("serializer", None)
        self.lookup = kwargs.pop("lookup", "id")
        if self.serializer is not None:
            assert issubclass(
                self.serializer, serializers.Serializer
            ), '"serializer" is not a valid serializer class'
            assert hasattr(
                self.serializer.Meta, "model"
            ), 'Class {serializer_class} missing "Meta.model" attribute'.format(
                serializer_class=self.serializer.__class__.__name__
            )
            kwargs["queryset"] = kwargs.get(
                "queryset", self.serializer.Meta.model.objects.all()
            )
            self.allow_insert = kwargs.pop("allow_insert", False)
            # kwargs["allow_null"] = kwargs.get("allow_null", self.serializer.Meta.model._meta.get_field(self.source).null)
        super().__init__(**kwargs)

    def use_pk_only_optimization(self):
        return False if self.serializer else True

    def to_representation(self, instance):
        if self.serializer:
            return self.serializer(instance, context=self.context).data
        return super().to_representation(instance)

    def to_internal_value(self, data):
        if isinstance(data, dict):
            instance = (
                self.get_queryset()
                .filter(**{self.lookup: data.get(self.lookup, None)})
                .first()
            )
            if self.allow_insert is True and len(data.keys()) and self.serializer:
                serialized_data = self.serializer(
                    instance=instance, data=data, context=self.context
                )
                if serialized_data.is_valid(raise_exception=ValueError):
                    instance = serialized_data.save()
            return instance
        return super().to_internal_value(data)

    def get_choices(self, cutoff=None):
        queryset = self.get_queryset()
        if queryset is None:
            return {}
        if cutoff is not None:
            queryset = queryset[:cutoff]

        return OrderedDict(
            [
                (
                    super(RelatedField, self).to_representation(item),
                    self.display_value(item),
                )
                for item in queryset
            ]
        )



from .base import BaseModelViewset
from .mixins import BulkDeleteMixin, GetUserLanguageMixin, TrigramSearchMixin
from ..parsers import MultipartJsonParser


from rest_framework.response import Response

from ..models import Media, MediaFolder
from ..serializers import MediaSerializer, MediaFolderSerializer, MediaListSerializer


class ParseMimeMixin(object):
    def parse_filter(self, filter):
        filter_name, value = filter.split("=")
        if filter_name == "mime_type":
            if value == "*/*":
                return "mime_type__isnull", False
            elif value.endswith("/*"):
                return "mime_type__startswith", value.split("/")[0]
        return filter_name, super().parse_qs_value(value)


class MediaFolderViewSet(
    GetUserLanguageMixin, ParseMimeMixin, TrigramSearchMixin, BaseModelViewset
):
    model = MediaFolder
    serializer_class = MediaFolderSerializer
    items_per_page = 18
    search_fields = ["name", "title", "alt_text"]

    def get_queryset(self):
        return self.model.objects.all()

    def get_serializer_context(self):
        return {**super().get_serializer_context(), "action": "list"}

    def get_mixed_response(self, request, *args, **kwargs):
        updir = kwargs.get("pk", None)

        parent_folder = MediaFolderSerializer(
            self.model.objects.filter(pk=updir).first()
        ).data
        folder_queryset = self.model.objects.filter(updir__pk=updir)
        media_queryset = (
            Media.objects.language(self.active_language)
            if request.GET.get("search", None)
            else Media.objects.all()
        ).filter(folder__pk=updir)

        folder_data = MediaFolderSerializer(
            folder_queryset, many=True, context={"request": request}
        ).data
        media_data = self.format_output(
            *self.handle_pagination_stack(media_queryset),
            SerializerClass=MediaListSerializer
        )
        return {
            "folders": folder_data,
            "media": media_data,
            "parent_folder": parent_folder,
        }

    def list(self, request, *args, **kwargs):
        return Response(self.get_mixed_response(request, *args, **kwargs))

    def retrieve(self, request, *args, **kwargs):
        return Response(self.get_mixed_response(request, *args, **kwargs))


class MediaViewSet(
    GetUserLanguageMixin,
    BulkDeleteMixin,
    ParseMimeMixin,
    TrigramSearchMixin,
    BaseModelViewset,
):

    queryset = Media.objects.all()
    serializer_class = MediaSerializer
    model = Media
    parser_classes = [MultipartJsonParser]



# Generated by Django 2.2.17 on 2021-12-15 12:47

from ..fields import JSONField
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("camomilla", "0004_auto_20210511_0937"),
    ]

    operations = [
        migrations.AddField(
            model_name="media",
            name="image_props",
            field=JSONField(default=dict, blank=True),
        ),
    ]



from django.db import models
from hvad.models import TranslatableModel, TranslatedFields


class BaseCategory(TranslatableModel):
    translations = TranslatedFields(
        title=models.CharField(max_length=200),
        description=models.TextField(blank=True, null=True, default=""),
        slug=models.SlugField(),
    )

    class Meta:
        abstract = True
        unique_together = [("title", "language_code")]
        verbose_name_plural = "categories"

    def __str__(self):
        return self.title


class Category(BaseCategory):
    translations = TranslatedFields()



from .base import BaseModelViewset
from .mixins import GetUserLanguageMixin, BulkDeleteMixin
from ..models import Category
from ..serializers import CategorySerializer
from ..permissions import CamomillaBasePermissions


class CategoryViewSet(GetUserLanguageMixin, BulkDeleteMixin, BaseModelViewset):

    queryset = Category.objects.all()
    serializer_class = CategorySerializer
    permission_classes = (CamomillaBasePermissions,)
    model = Category



from django.conf import settings
from hvad.contrib.restframework import TranslationsMixin
from rest_framework import serializers
from django.utils import translation


class LangInfoMixin(metaclass=serializers.SerializerMetaclass):
    lang_info = serializers.SerializerMethodField("get_lang_info", read_only=True)

    def get_lang_info(self, obj, *args, **kwargs):
        languages = []
        for key, language in settings.LANGUAGES:
            languages.append({"id": key, "name": language})
        return {
            "default": settings.LANGUAGE_CODE,
            "active": translation.get_language(),
            "translated_in": obj.get_available_languages(),
            "site_languages": languages,
        }

    def get_default_field_names(self, *args):
        field_names = super().get_default_field_names(*args)
        self.action = getattr(
            self, "action", self.context and self.context.get("action", "list")
        )
        if self.action != "retrieve":
            return [f for f in field_names if f != "lang_info"]
        return field_names


class TranslationSetMixin(TranslationsMixin):
    def get_default_field_names(self, *args):
        field_names = super(TranslationsMixin, self).get_default_field_names(*args)
        self.action = getattr(
            self, "action", self.context and self.context.get("action", "list")
        )
        if self.action != "list":
            field_names += [self.Meta.model._meta.translations_accessor]
        return field_names


class SetupEagerLoadingMixin:
    @staticmethod
    def setup_eager_loading(queryset):
        return queryset



from .base import BaseModelViewset
from .mixins import GetUserLanguageMixin, BulkDeleteMixin

from ..models import Article
from ..serializers import ArticleSerializer
from ..permissions import CamomillaBasePermissions


class ArticleViewSet(GetUserLanguageMixin, BulkDeleteMixin, BaseModelViewset):

    queryset = Article.objects.all()
    serializer_class = ArticleSerializer
    permission_classes = (CamomillaBasePermissions,)
    search_fields = ["title", "identifier", "content", "permalink"]
    model = Article



# Generated by Django 2.2.25 on 2022-01-03 18:45

from django.db import migrations, models
from ..models import Media


def forwards_func(apps, schema_editor):
    db_alias = schema_editor.connection.alias
    for m in Media.objects.using(db_alias).all():
        m.save()


def reverse_func(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ("camomilla", "0005_media_image_props"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="media",
            name="is_image",
        ),
        migrations.AddField(
            model_name="media",
            name="mime_type",
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
        migrations.RunPython(forwards_func, reverse_func),
    ]



import pytest
from django.contrib.auth.models import User
from rest_framework.test import APIClient

from camomilla.models import Tag, Category

client = APIClient()


def login_superuser():
    User.objects.create_superuser("admin", "myemail@test.com", "adminadmin")
    response = client.post(
        "/api/camomilla/token-auth/", {"username": "admin", "password": "adminadmin"}
    )
    return response.json()["token"]


@pytest.mark.django_db
def test_create_tag_no_access():
    response = client.post("/api/camomilla/tags/", {"title": "First tag"})
    assert response.json()["detail"] == "Authentication credentials were not provided."
    assert response.status_code == 403


@pytest.mark.django_db
def test_crud_tag():
    ## Create
    token = login_superuser()
    client.credentials(HTTP_AUTHORIZATION="Token " + token)
    response = client.post("/api/camomilla/tags/", {"title": "Primo tag"})
    assert response.json()["title"] == "Primo tag"
    assert response.json()["language_code"] == "it"

    assert len(Tag.objects.all()) == 1
    assert response.status_code == 201

    ## Create another with a different language
    response = client.post(
        "/api/camomilla/tags/", {"title": "Second tag", "language_code": "en"}
    )
    assert response.json()["title"] == "Second tag"
    assert response.json()["language_code"] == "en"

    assert len(Tag.objects.all()) == 2

    assert response.status_code == 201

    ## Translate the second one in italian
    response = client.patch(
        "/api/camomilla/tags/2/", {"title": "Secondo tag", "language_code": "it"}
    )
    assert response.json()["title"] == "Secondo tag"
    assert response.json()["language_code"] == "it"

    assert len(Tag.objects.all()) == 2

    assert response.status_code == 200

    ## Get the tags in 🇮🇹
    response = client.get("/api/camomilla/tags/")

    assert response.json()[1]["title"] == "Secondo tag"
    assert response.json()[1]["language_code"] == "it"

    ## Get the tags in 🇬🇧 with fallbacks!
    response = client.get("/api/camomilla/tags/?language=en")

    assert response.json()[1]["title"] == "Second tag"
    assert response.json()[1]["language_code"] == "en"

    ## Delete the tag
    response = client.delete("/api/camomilla/tags/2/")

    assert len(Tag.objects.all()) == 1

    assert response.status_code == 204



import json
import six

from django.http.multipartparser import MultiPartParser as DjangoMultiPartParser
from django.http.multipartparser import MultiPartParserError
from django.conf import settings
from rest_framework import parsers
from rest_framework.exceptions import ParseError


def set_key(data, key, val):
    if isinstance(data, list):
        key = int(key)
        if key < len(data):
            data[key] = val
        else:
            data.append(val)
        return data
    data[key] = val
    return data


def get_key(data, key, default):
    if isinstance(data, list):
        try:
            return data[int(key)]
        except IndexError:
            return default
    return data.get(key, default)


def compile_payload(data, path, value):
    key = path.pop(0)
    if not len(path):
        return set_key(data, key, value)
    default = [] if path[0].isdigit() else {}
    return set_key(data, key, compile_payload(get_key(data, key, default), path, value))


class MultipartJsonParser(parsers.BaseParser):

    media_type = "multipart/form-data"

    def parse(self, stream, media_type=None, parser_context=None):
        parser_context = parser_context or {}
        request = parser_context["request"]
        encoding = parser_context.get("encoding", settings.DEFAULT_CHARSET)
        meta = request.META.copy()
        meta["CONTENT_TYPE"] = media_type
        upload_handlers = request.upload_handlers

        try:
            parser = DjangoMultiPartParser(meta, stream, upload_handlers, encoding)
            data, files = parser.parse()
            result = parsers.DataAndFiles(data, files)
            data = {}
            data = json.loads(result.data["data"])
            for key, value in result.files.items():
                data = compile_payload(data, key.split("."), value)
            return data
        except MultiPartParserError as exc:
            raise ParseError("Multipart form parse error - %s" % six.text_type(exc))



from rest_framework import status
from rest_framework.decorators import action
from rest_framework.response import Response
from .pagination import *
from ...permissions import CamomillaBasePermissions
from django.utils import translation


class GetUserLanguageMixin(object):
    def _get_user_language(self, request):
        self.active_language = request.GET.get(
            "language",
            request.GET.get(
                "language_code", translation.get_language_from_request(request)
            ),
        )
        self.language_fallbacks = True
        if (
            len(self.active_language.split("-")) == 2
            and self.active_language.split("-")[0] == "nofallbacks"
        ):
            self.language_fallbacks = False
            self.active_language = self.active_language.split("-")[1]
        translation.activate(self.active_language)
        return self.active_language

    def initialize_request(self, request, *args, **kwargs):
        self._get_user_language(request)
        return super().initialize_request(request, *args, **kwargs)

    def get_queryset(self):
        return (
            self.model.objects.language(self.active_language).fallbacks().all()
            if self.language_fallbacks
            else self.model.objects.language(self.active_language).all()
        )


class OptimViewMixin:
    def get_serializer_class(self):
        if hasattr(self, "action_serializers"):
            if self.action in self.action_serializers:
                return self.action_serializers[self.action]
        return super().get_serializer_class()

    def get_serializer_context(self):
        return {"request": self.request, "action": self.action}

    def get_queryset(self):
        queryset = super().get_queryset()
        serializer = self.get_serializer_class()
        if hasattr(serializer, "setup_eager_loading"):
            queryset = self.get_serializer_class().setup_eager_loading(queryset)
        return queryset


class BulkDeleteMixin(object):
    @action(
        detail=False, methods=["post"], permission_classes=(CamomillaBasePermissions,)
    )
    def bulk_delete(self, request):
        try:
            self.model.objects.filter(pk__in=request.data).delete()
            return Response(
                {"detail": "Eliminazione multipla andata a buon fine"},
                status=status.HTTP_200_OK,
            )
        except Exception:
            return Response(
                {"detail": "Eliminazione multipla non riuscita"},
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )



from .base import BaseModelViewset
from .mixins import GetUserLanguageMixin, BulkDeleteMixin
from ..models import Tag
from ..serializers import TagSerializer
from ..permissions import CamomillaBasePermissions


class TagViewSet(GetUserLanguageMixin, BulkDeleteMixin, BaseModelViewset):

    queryset = Tag.objects.all()
    serializer_class = TagSerializer
    permission_classes = (CamomillaBasePermissions,)
    model = Tag



from .base import BaseModelViewset
from .mixins import GetUserLanguageMixin, BulkDeleteMixin
from ..models import Page
from ..serializers import PageSerializer
from ..permissions import CamomillaBasePermissions


class PageViewSet(GetUserLanguageMixin, BulkDeleteMixin, BaseModelViewset):

    queryset = Page.objects.all()
    serializer_class = PageSerializer
    permission_classes = (CamomillaBasePermissions,)
    model = Page



from django.db import models

from django.urls import reverse

from hvad.models import TranslatableModel, TranslatedFields

from djsuperadmin.mixins import DjSuperAdminMixin


class BaseContent(DjSuperAdminMixin, TranslatableModel):
    identifier = models.CharField(max_length=200)
    translations = TranslatedFields(
        title=models.CharField(max_length=200),
        subtitle=models.CharField(max_length=200, blank=True, null=True, default=""),
        permalink=models.CharField(max_length=200, blank=False, null=True),
        content=models.TextField(default=""),
    )
    page = models.ForeignKey(
        "camomilla.Page",
        blank=False,
        null=True,
        on_delete=models.SET_NULL,
        related_name="contents",
    )

    @property
    def superadmin_get_url(self):
        return reverse("camomilla-content-djsuperadmin", kwargs={"pk": self.pk})

    @property
    def superadmin_patch_url(self):
        return reverse("camomilla-content-djsuperadmin", kwargs={"pk": self.pk})

    class Meta:
        abstract = True
        unique_together = [("page", "identifier")]

    def __str__(self):
        return self.identifier


class Content(BaseContent):
    translations = TranslatedFields()



from django.db import models
from hvad.models import TranslatableModel, TranslatedFields


class BaseTag(TranslatableModel):
    translations = TranslatedFields(
        title=models.CharField(max_length=200),
    )

    class Meta:
        abstract = True
        unique_together = [("title", "language_code")]

    def __str__(self):
        return self.title


class Tag(BaseTag):
    translations = TranslatedFields()



from ..models import Article, Category, Tag
from .fields import RelatedField
from .media import MediaSerializer
from .base import BaseTranslatableModelSerializer


class TagSerializer(BaseTranslatableModelSerializer):
    class Meta:
        model = Tag
        fields = "__all__"


class CategorySerializer(BaseTranslatableModelSerializer):
    class Meta:
        model = Category
        fields = "__all__"


class ArticleSerializer(BaseTranslatableModelSerializer):

    highlight_image = RelatedField(
        serializer=MediaSerializer, required=False, allow_null=True
    )
    tags = RelatedField(
        serializer=TagSerializer, many=True, required=False, allow_null=True
    )
    categories = RelatedField(
        serializer=CategorySerializer, many=True, required=False, allow_null=True
    )
    og_image = RelatedField(serializer=MediaSerializer, required=False, allow_null=True)

    class Meta:
        model = Article
        fields = "__all__"



from ..models import Content, Page
from .fields import RelatedField
from .media import MediaSerializer
from .base import BaseTranslatableModelSerializer


class ContentSerializer(BaseTranslatableModelSerializer):
    class Meta:
        model = Content
        fields = "__all__"


class PageSerializer(BaseTranslatableModelSerializer):
    og_image = RelatedField(serializer=MediaSerializer, required=False, allow_null=True)
    contents = RelatedField(
        serializer=ContentSerializer, many=True, required=False, allow_null=True
    )

    class Meta:
        model = Page
        fields = "__all__"



from ..mixins import LangInfoMixin, SetupEagerLoadingMixin, TranslationSetMixin
from rest_framework import serializers
from hvad.contrib.restframework import TranslatableModelSerializer
from django.db import models
from ..fields import FileField, ImageField, RelatedField


class BaseModelSerializer(SetupEagerLoadingMixin, serializers.ModelSerializer):
    serializer_field_mapping = {
        **serializers.ModelSerializer.serializer_field_mapping,
        models.FileField: FileField,
        models.ImageField: ImageField,
    }
    serializer_related_field = RelatedField


class BaseTranslatableModelSerializer(
    LangInfoMixin, TranslationSetMixin, TranslatableModelSerializer
):
    serializer_field_mapping = {
        **TranslatableModelSerializer.serializer_field_mapping,
        models.FileField: FileField,
        models.ImageField: ImageField,
    }
    serializer_related_field = RelatedField



# Generated by Django 2.2.6 on 2019-10-10 17:00

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Category",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "categories",
                "abstract": False,
            },
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("_plain_manager", django.db.models.manager.Manager()),
            ],
        ),
        migrations.CreateModel(
            name="Media",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("file", models.FileField(upload_to="")),
                (
                    "thumbnail",
                    models.ImageField(
                        blank=True, max_length=500, null=True, upload_to="thumbnails"
                    ),
                ),
                ("created", models.DateTimeField(auto_now=True)),
                ("name", models.CharField(blank=True, max_length=200, null=True)),
                ("size", models.IntegerField(blank=True, default=0, null=True)),
                ("is_image", models.BooleanField(default=False)),
            ],
            options={
                "ordering": ["-pk"],
            },
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("_plain_manager", django.db.models.manager.Manager()),
            ],
        ),
        migrations.CreateModel(
            name="MediaFolder",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("slug", models.SlugField()),
                ("creation_date", models.DateTimeField(auto_now_add=True)),
                ("last_modified", models.DateTimeField(auto_now=True)),
                (
                    "icon",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="camomilla.Media",
                        verbose_name="Image cover",
                    ),
                ),
                (
                    "updir",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="camomilla.MediaFolder",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("_plain_manager", django.db.models.manager.Manager()),
            ],
        ),
        migrations.CreateModel(
            name="Page",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("identifier", models.CharField(max_length=200, unique=True)),
                (
                    "og_image",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="camomilla_page_related",
                        to="camomilla.Media",
                    ),
                ),
            ],
            options={
                "verbose_name": "Page",
                "verbose_name_plural": "Pages",
                "abstract": False,
            },
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("_plain_manager", django.db.models.manager.Manager()),
            ],
        ),
        migrations.CreateModel(
            name="Tag",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("_plain_manager", django.db.models.manager.Manager()),
            ],
        ),
        migrations.AddField(
            model_name="media",
            name="folder",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="media_folder",
                to="camomilla.MediaFolder",
            ),
        ),
        migrations.CreateModel(
            name="Content",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("identifier", models.CharField(max_length=200)),
                (
                    "page",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="contents",
                        to="camomilla.Page",
                    ),
                ),
            ],
            options={
                "abstract": False,
                "unique_together": {("page", "identifier")},
            },
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("_plain_manager", django.db.models.manager.Manager()),
            ],
        ),
        migrations.CreateModel(
            name="Article",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("trash", models.BooleanField(default=False)),
                ("identifier", models.CharField(max_length=200, unique=True)),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("PUB", "Published"),
                            ("DRF", "Draft"),
                            ("TRS", "Trash"),
                            ("PLA", "Planned"),
                        ],
                        default="DRF",
                        max_length=3,
                    ),
                ),
                ("date", models.DateTimeField(auto_now=True)),
                ("pubblication_date", models.DateTimeField(blank=True, null=True)),
                (
                    "author",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "categories",
                    models.ManyToManyField(blank=True, to="camomilla.Category"),
                ),
                (
                    "highlight_image",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="camomilla.Media",
                    ),
                ),
                (
                    "og_image",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="camomilla_article_related",
                        to="camomilla.Media",
                    ),
                ),
                ("tags", models.ManyToManyField(blank=True, to="camomilla.Tag")),
            ],
            options={
                "abstract": False,
            },
            managers=[
                ("trashmanager", django.db.models.manager.Manager()),
                ("_plain_manager", django.db.models.manager.Manager()),
            ],
        ),
        migrations.CreateModel(
            name="TagTranslation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=200)),
                ("language_code", models.CharField(db_index=True, max_length=15)),
                (
                    "master",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="camomilla.Tag",
                    ),
                ),
            ],
            options={
                "db_table": "camomilla_tag_translation",
                "db_tablespace": "",
                "abstract": False,
                "managed": True,
                "default_permissions": (),
                "unique_together": {
                    ("language_code", "master"),
                    ("title", "language_code"),
                },
            },
        ),
        migrations.CreateModel(
            name="PageTranslation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=200)),
                ("description", models.TextField(blank=True, default="", null=True)),
                ("permalink", models.CharField(blank=True, max_length=200)),
                ("og_description", models.TextField(blank=True, default="", null=True)),
                (
                    "og_title",
                    models.CharField(blank=True, default="", max_length=200, null=True),
                ),
                (
                    "og_type",
                    models.CharField(blank=True, default="", max_length=200, null=True),
                ),
                (
                    "og_url",
                    models.CharField(blank=True, default="", max_length=200, null=True),
                ),
                (
                    "canonical",
                    models.CharField(blank=True, default="", max_length=200, null=True),
                ),
                ("language_code", models.CharField(db_index=True, max_length=15)),
                (
                    "master",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="camomilla.Page",
                    ),
                ),
            ],
            options={
                "db_table": "camomilla_page_translation",
                "db_tablespace": "",
                "abstract": False,
                "managed": True,
                "default_permissions": (),
                "unique_together": {("language_code", "master")},
            },
        ),
        migrations.CreateModel(
            name="MediaTranslation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("alt_text", models.CharField(blank=True, max_length=200, null=True)),
                ("title", models.CharField(blank=True, max_length=200, null=True)),
                ("description", models.TextField(blank=True, null=True)),
                ("language_code", models.CharField(db_index=True, max_length=15)),
                (
                    "master",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="camomilla.Media",
                    ),
                ),
            ],
            options={
                "db_table": "camomilla_media_translation",
                "db_tablespace": "",
                "abstract": False,
                "managed": True,
                "default_permissions": (),
                "unique_together": {("language_code", "master")},
            },
        ),
        migrations.CreateModel(
            name="MediaFolderTranslation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "description",
                    models.CharField(blank=True, max_length=200, null=True),
                ),
                ("title", models.CharField(blank=True, max_length=200, null=True)),
                ("language_code", models.CharField(db_index=True, max_length=15)),
                (
                    "master",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="camomilla.MediaFolder",
                    ),
                ),
            ],
            options={
                "db_table": "camomilla_mediafolder_translation",
                "db_tablespace": "",
                "abstract": False,
                "managed": True,
                "default_permissions": (),
                "unique_together": {("language_code", "master")},
            },
        ),
        migrations.CreateModel(
            name="ContentTranslation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=200)),
                (
                    "subtitle",
                    models.CharField(blank=True, default="", max_length=200, null=True),
                ),
                ("permalink", models.CharField(blank=True, max_length=200, null=True)),
                ("content", models.TextField()),
                ("language_code", models.CharField(db_index=True, max_length=15)),
                (
                    "master",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="camomilla.Content",
                    ),
                ),
            ],
            options={
                "db_table": "camomilla_content_translation",
                "db_tablespace": "",
                "abstract": False,
                "managed": True,
                "default_permissions": (),
                "unique_together": {("language_code", "master")},
            },
        ),
        migrations.CreateModel(
            name="CategoryTranslation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=200)),
                ("description", models.TextField(blank=True, default="", null=True)),
                ("slug", models.SlugField()),
                ("language_code", models.CharField(db_index=True, max_length=15)),
                (
                    "master",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="camomilla.Category",
                    ),
                ),
            ],
            options={
                "db_table": "camomilla_category_translation",
                "db_tablespace": "",
                "abstract": False,
                "managed": True,
                "default_permissions": (),
                "unique_together": {
                    ("language_code", "master"),
                    ("title", "language_code"),
                },
            },
        ),
        migrations.CreateModel(
            name="ArticleTranslation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=200)),
                ("description", models.TextField(blank=True, default="", null=True)),
                ("permalink", models.CharField(blank=True, max_length=200)),
                ("og_description", models.TextField(blank=True, default="", null=True)),
                (
                    "og_title",
                    models.CharField(blank=True, default="", max_length=200, null=True),
                ),
                (
                    "og_type",
                    models.CharField(blank=True, default="", max_length=200, null=True),
                ),
                (
                    "og_url",
                    models.CharField(blank=True, default="", max_length=200, null=True),
                ),
                (
                    "canonical",
                    models.CharField(blank=True, default="", max_length=200, null=True),
                ),
                ("content", models.TextField()),
                ("language_code", models.CharField(db_index=True, max_length=15)),
                (
                    "master",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="camomilla.Article",
                    ),
                ),
            ],
            options={
                "db_table": "camomilla_article_translation",
                "db_tablespace": "",
                "abstract": False,
                "managed": True,
                "default_permissions": (),
                "unique_together": {
                    ("permalink", "language_code"),
                    ("language_code", "master"),
                },
            },
        ),
    ]



from rest_framework.fields import FileField as DRFFileField
from rest_framework.fields import ImageField as DRFImageField
from rest_framework.fields import Field


class SafeFileFieldMixin(Field):
    def to_internal_value(self, data):
        current = getattr(self.parent.instance, self.field_name, None)
        if (
            isinstance(data, str)
            and current
            and data.endswith(getattr(current, "url", ""))
        ):
            return current
        return super().to_internal_value(data)


class FileField(SafeFileFieldMixin, DRFFileField):
    pass


class ImageField(SafeFileFieldMixin, DRFImageField):
    pass



from django.contrib import admin

from django import forms
from django.http import HttpResponse

from .models import Article, Tag, Category, Content, Media, Page, MediaFolder
from ckeditor_uploader.widgets import CKEditorUploadingWidget


from hvad.admin import TranslatableAdmin
from hvad.forms import TranslatableModelForm

from django.conf import settings
from django.utils.translation import ugettext_lazy as _


class UserProfileAdmin(admin.ModelAdmin):
    pass


class ArticleAdminForm(TranslatableModelForm):
    content = forms.CharField(widget=CKEditorUploadingWidget())

    class Meta:
        model = Article
        exclude = ("slug",)


class ArticleAdmin(TranslatableAdmin):
    filter_horizontal = ("tags", "categories")
    form = ArticleAdminForm


class TagAdmin(TranslatableAdmin):
    pass


class CategoryAdmin(TranslatableAdmin):
    pass


class MediaFolderAdmin(TranslatableAdmin):
    pass


class ContentAdminForm(TranslatableModelForm):
    exclude = ("permalink",)
    content = forms.CharField(widget=CKEditorUploadingWidget())

    class Meta:
        model = Content


class ContentAdmin(TranslatableAdmin):
    form = ContentAdminForm


class MediaAdmin(TranslatableAdmin):
    exclude = (
        "thumbnail",
        "size",
        "image_props",
    )
    readonly_fields = ("image_preview", "image_thumb_preview")
    list_display = (
        "__str__",
        "name",
        "image_thumb_preview",
    )

    def response_add(self, request, obj):
        if request.GET.get("_popup", ""):
            return HttpResponse(
                """
               <script type="text/javascript">
                  opener.dismissAddRelatedObjectPopup(window, %s, '%s');
               </script>"""
                % (obj.id, obj.json_repr)
            )
        else:
            return super(MediaAdmin, self).response_add(request, obj)


class PageAdmin(TranslatableAdmin):
    exclude = ("permalink",)


admin.site.register(Article, ArticleAdmin)
admin.site.register(Category, CategoryAdmin)
admin.site.register(MediaFolder, MediaFolderAdmin)
admin.site.register(Tag, TagAdmin)
admin.site.register(Content, ContentAdmin)
admin.site.register(Media, MediaAdmin)
admin.site.register(Page, PageAdmin)

admin.site.index_title = "{0} {1}".format(
    _("Administration panel for"), getattr(settings, "PROJECT_TITLE", "Camomilla")
)



from django.conf import settings
from django.db import models

from django.utils.translation import ugettext_lazy as _

from hvad.models import TranslatedFields

from .mixins import SeoMixin


CONTENT_STATUS = (
    ("PUB", _("Published")),
    ("DRF", _("Draft")),
    ("TRS", _("Trash")),
    ("PLA", _("Planned")),
)


class BaseArticle(SeoMixin):

    seo_attr = "permalink"

    identifier = models.CharField(max_length=200, unique=True)
    translations = TranslatedFields(
        content=models.TextField(default=""),
        permalink=models.SlugField(max_length=200, blank=False),
    )
    author = models.ForeignKey(
        settings.AUTH_USER_MODEL, blank=True, null=True, on_delete=models.SET_NULL
    )
    status = models.CharField(
        max_length=3,
        choices=CONTENT_STATUS,
        default="DRF",
    )
    highlight_image = models.ForeignKey(
        "camomilla.Media", blank=True, null=True, on_delete=models.SET_NULL
    )
    date = models.DateTimeField(auto_now=True)
    pubblication_date = models.DateTimeField(null=True, blank=True)
    tags = models.ManyToManyField("camomilla.Tag", blank=True)
    categories = models.ManyToManyField("camomilla.Category", blank=True)

    class Meta:
        abstract = True
        unique_together = [("permalink", "language_code")]

    def save(self, *args, **kwargs):
        import uuid

        if not self.identifier:
            self.identifier = "{0}".format(str(uuid.uuid4()))
        super(BaseArticle, self).save(*args, **kwargs)

    def __str__(self):
        return self.title


class Article(BaseArticle):
    translations = TranslatedFields()



from django.urls import include, path
from django.conf.urls.i18n import i18n_patterns


def dummy_view(request):
    from django.http import HttpResponse

    return HttpResponse("Here's the text of the Web page.")


urlpatterns = [
    path("articles/<slug:title>", dummy_view, name="article-detail"),
    path("api/camomilla/", include("camomilla.urls")),
]


urlpatterns += i18n_patterns(
    path("about", dummy_view, name="about"),
    path("international_articles/<slug:title>", dummy_view, name="int-article-detail"),
    prefix_default_language=False,
)



# Generated by Django 2.2.6 on 2020-02-14 11:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("camomilla", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="articletranslation",
            name="content",
            field=models.TextField(default=""),
        ),
        migrations.AlterField(
            model_name="articletranslation",
            name="permalink",
            field=models.CharField(max_length=200),
        ),
        migrations.AlterField(
            model_name="contenttranslation",
            name="content",
            field=models.TextField(default=""),
        ),
        migrations.AlterField(
            model_name="contenttranslation",
            name="permalink",
            field=models.CharField(max_length=200, null=True),
        ),
    ]



from django.core.management.base import BaseCommand
from camomilla.models import Media


class Command(BaseCommand):

    help = "Regenerates all the thumbnail"

    def handle(self, *args, **options):
        for media in Media.objects.all():
            media.regenerate_thumbnail()
            media.save()
            self.stdout.write(
                self.style.SUCCESS(
                    "Successfully regenerated thumbnail for {0}".format(media.file.url)
                )
            )



from rest_framework import serializers

from ..models import Media, MediaFolder
from ..storages import OverwriteStorage
from .base import BaseTranslatableModelSerializer


class MediaListSerializer(BaseTranslatableModelSerializer):
    is_image = serializers.SerializerMethodField("get_is_image")

    def get_is_image(self, obj):
        return obj.is_image

    class Meta:
        model = Media
        fields = "__all__"


class MediaSerializer(BaseTranslatableModelSerializer):
    links = serializers.SerializerMethodField("get_linked_instances")
    is_image = serializers.SerializerMethodField("get_is_image")

    class Meta:
        model = Media
        fields = "__all__"

    def get_linked_instances(self, obj):
        result = []
        links = obj.get_foreign_fields()
        for link in links:
            manager = getattr(obj, link)
            if hasattr(manager, "language"):
                manager = manager.language()
            for item in manager.all():
                if item.__class__.__name__ != "MediaTranslation":
                    result.append(
                        {
                            "model": item.__class__.__name__,
                            "name": item.__str__(),
                            "id": item.pk,
                        }
                    )
        return result

    def update(self, instance, data):
        same_url = self.initial_data.get("same_url", False)
        if same_url:
            new_file = data.pop("file", None)
            if new_file:
                instance = super().update(instance, data)
                instance.file.storage = OverwriteStorage()
                instance.file.save(instance.file.name, new_file, save=True)
                return instance
        return super().update(instance, data)

    def get_is_image(self, obj):
        return obj.is_image


class MediaFolderSerializer(BaseTranslatableModelSerializer):
    icon = MediaSerializer(read_only=True)

    class Meta:
        model = MediaFolder
        fields = "__all__"



# Generated by Django 2.2.17 on 2021-05-11 09:37

from django.db import migrations
import django.db.models.manager


class Migration(migrations.Migration):

    dependencies = [
        ("camomilla", "0003_auto_20210130_1610"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="article",
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("_plain_manager", django.db.models.manager.Manager()),
            ],
        ),
        migrations.RemoveField(
            model_name="article",
            name="trash",
        ),
    ]



import os
import mock

from django.http import Http404
from django.test import TestCase
from django.test import RequestFactory
from django.utils.translation import activate, get_language

from camomilla.models import Article
from camomilla.templatetags.camomilla_filters import filter_content, alternate_urls

from requests import RequestException


class CamomillaFiltersTestCase(TestCase):
    def setUp(self):
        pass

    def test_create_articles_with_auto_identifier(self):
        article1 = Article.objects.create(permalink="article-1")
        article2 = Article.objects.create(permalink="article-2")
        self.assertNotEqual(article1.identifier, article2.identifier)



from django.db import models
from hvad.models import TranslatableModel, TranslatedFields

from django.utils.translation import ugettext_lazy as _
from django.utils.text import slugify

from ...utils import get_seo_model

from djlotrek.utils import alternate_seo_url_with_object


class SeoMixin(TranslatableModel):

    seo_attr = "identifier"

    translations = TranslatedFields(
        title=models.CharField(max_length=200),
        description=models.TextField(blank=True, null=True, default=""),
        permalink=models.CharField(max_length=200, blank=True),
        og_description=models.TextField(blank=True, null=True, default=""),
        og_title=models.CharField(max_length=200, blank=True, null=True, default=""),
        og_type=models.CharField(max_length=200, blank=True, null=True, default=""),
        og_url=models.CharField(max_length=200, blank=True, null=True, default=""),
        canonical=models.CharField(max_length=200, blank=True, null=True, default=""),
    )
    og_image = models.ForeignKey(
        "camomilla.Media",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name="%(app_label)s_%(class)s_related",
    )

    @classmethod
    def get(model, request, **kwargs):
        return get_seo_model(request, model, **kwargs)

    def alternate_urls(self, request):
        return alternate_seo_url_with_object(
            request, self.__class__, permalink=self.permalink
        )

    class Meta:
        abstract = True


class SlugMixin(object):

    slug_attr = "title"

    def get_slug(self):
        return self.slug

    get_slug.short_description = _("Slug")

    def save(self, *args, **kwargs):
        self.slug = slugify(getattr(self, self.slug_attr))
        super(SlugMixin, self).save(*args, **kwargs)
